<?php
/**
 * Copyright 2013 Drexel University
 */
require_once __dir__ . "/irods/RODSKeyValPair.class.php";
require_once __dir__ . "/irods/ProdsRule.class.php";
require_once __dir__ . "/irods/RODSMeta.class.php";
require_once __dir__ . "/irods/ProdsTicket.class.php";
require_once __dir__ . "/irods/packet/RODSPacket.class.php";
require_once __dir__ . "/irods/packet/RP_ModAVUMetadataInp.class.php";
require_once __dir__ . "/irods/RODSException.class.php";
require_once __dir__ . "/irods/packet/RP_DataObjCopyInp.class.php";
require_once __dir__ . "/irods/packet/RP_CollOprStat.class.php";
require_once __dir__ . "/irods/packet/RP_dataObjCloseInp.class.php";
require_once __dir__ . "/irods/packet/RP_dataObjWriteInp.class.php";
require_once __dir__ . "/irods/packet/RP_CollInp.class.php";
require_once __dir__ . "/irods/packet/RP_DataObjInp.class.php";
require_once __dir__ . "/irods/packet/RP_SqlResult.class.php";
require_once __dir__ . "/irods/packet/RP_GenQueryOut.class.php";
require_once __dir__ . "/irods/packet/RP_GenQueryInp.class.php";
require_once __dir__ . "/irods/packet/RP_InxValPair.class.php";
require_once __dir__ . "/irods/packet/RP_KeyValPair.class.php";
require_once __dir__ . "/irods/packet/RP_InxIvalPair.class.php";
require_once __dir__ . "/irods/RODSQueryCondition.class.php";
require_once __dir__ . "/irods/packet/RP_authResponseInp.class.php";
require_once __dir__ . "/irods/packet/RP_authRequestOut.class.php";
require_once __dir__ . "/irods/packet/RP_Version.class.php";
require_once __dir__ . "/irods/packet/RP_MsgHeader.class.php";
require_once __dir__ . "/irods/packet/RP_MsParam.class.php";
require_once __dir__ . "/irods/packet/RP_fileLseekInp.class.php";
require_once __dir__ . "/irods/packet/RP_fileLseekOut.class.php";
require_once __dir__ . "/irods/packet/RP_dataObjReadInp.class.php";
require_once __dir__ . "/irods/packet/RP_ticketAdminInp.class.php";
require_once __dir__ . "/irods/packet/RP_STR.class.php";
require_once __dir__ . "/irods/packet/RP_BinBytesBuf.class.php";
require_once __dir__ . "/irods/packet/RP_MsParamArray.class.php";
require_once __dir__ . "/irods/packet/RP_RHostAddr.class.php";
require_once __dir__ . "/irods/packet/RP_ExecMyRuleInp.class.php";
require_once __dir__ . "/irods/RODSMessage.class.php";
require_once __dir__ . "/irods/packet/RP_StartupPack.class.php";
require_once __dir__ . "/irods/RODSFileStats.class.php";
require_once __dir__ . "/irods/RODSGenQueResults.class.php";
require_once __dir__ . "/irods/RODSGenQueConds.class.php";
require_once __dir__ . "/irods/RODSGenQueSelFlds.class.php";
require_once __dir__ . "/irods/RODSConn.class.php";
require_once __dir__ . "/irods/RODSConnManager.class.php";
require_once __dir__ . "/irods/ProdsPath.class.php";
require_once __dir__ . "/irods/ProdsFile.class.php";
require_once __dir__ . "/irods/ProdsDir.class.php";
require_once __dir__ . "/irods/RODSAccount.class.php";

function getCustomAccount()
{
    global $wgUser;
    if ($wgUser->mId == 0) {
        return null;
    }
    $dbr = wfGetDB(DB_SLAVE);
    $returnValue = $dbr->selectRow(
        array("irods_users"),
        array("irods_user_name, irods_user_password, irods_user_resource, irods_user_zone"),
        array("wiki_user_id" => $wgUser->mId, "irods_custom_account_active" => 1)
    );
    if (isset($returnValue->irods_user_name) && isset($returnValue->irods_user_password)) {
        return $returnValue;
    } else {
        return null;
    }
}
// creates a global irods connection that could be used by all functions
function getAccount($forceDefault = false)
{
    global $configIrodsHost, $configIrodsPort, $configIrodsUser, $configIrodsPass,
    $configIrodsResource, $configIrodsZone;
    $customInfo = getCustomAccount();
    if (!$forceDefault && isset($customInfo)) {
        $account = new RODSAccount(
            $configIrodsHost,
            $configIrodsPort,
            $customInfo->irods_user_name,
            $customInfo->irods_user_password,
            $customInfo->irods_user_zone,
            $customInfo->irods_user_resource
        );
        return $account;
    }
    $account = new RODSAccount(
        $configIrodsHost,
        $configIrodsPort,
        $configIrodsUser,
        $configIrodsPass,
        $configIrodsZone,
        $configIrodsResource
    );
    return $account;
}
function get_user_rights_on_irods_file($path, $user)
{
    $myAccount = getAccount(true);
    $myfile = new ProdsFile($myAccount, $path); // creates irods file object
    if (!$myfile) {
        throw new RODSException('Bad file or account');
    }
    $rights = $myfile->getACL();
    if ($rights) {
        $rightsSize = sizeof($rights);
        for ($i = 0; $i < $rightsSize; $i++) {
            if (array_key_exists($i, $rights) && array_key_exists('COL_USER_NAME', $rights[$i]) && $rights[$i]->COL_USER_NAME == $user) {
                return $rights[$i];
            }
        }
    } else {
        throw new RODSException("Bad Access Control List.");
    }
}
// Small shortcut for ensuring $path does not end in a traling slash.
function buildProdsDirectory($path, $account, $verify = true)
{
    if ($path !== '/') {
        $path = rtrim($path, '/');
    }
    $newDirectory = new ProdsDir($account, $path, $verify);
    return $newDirectory;
}
// $parentDirectory is a ProdsDir object representing the parent dir.
// $newDirectoryName is the desired name for the new directory.
function makeIrodsChildDirectory($parentDirectory, $newDirectoryName)
{
    try {
        $childDirectory = $parentDirectory->mkdir($newDirectoryName); // creates dir
    } catch (Exception $e) {
        $errorCode = (int) $e->getCode();
        switch ($errorCode) {
            case -809000:
                $childDirectory = buildProdsDirectory(
                    "{$parentDirectory->path_str}/$newDirectoryName",
                    $parentDirectory->account,
                    false
                );
                break;
            case -818000:
                die("User not allowed to create iRODS collections.");
                break;
            default:
                throw $e;
        }
    }
    return $childDirectory;
}
// Shortcut for creating a new ProdsDir object and creating a child directory.
// $account argument is available for use when an account object has already been created.
function makeIrodsDirectory($path, $newDirectoryName, $account = null, $verify = true) {
    if (!isset($account)) {
        $account = getAccount();
    }
    $parentDirectory = buildProdsDirectory($path, $account, $verify);
    $childDirectory = makeIrodsChildDirectory($parentDirectory, $newDirectoryName);
    return $childDirectory;
}
function uploadFileOnIrods(&$src_path, $dest_path)
{
    $account = getAccount();
    // set running time for PHP to 0 = no limit
    set_time_limit(0);
    //    $file_to_be_uploaded = file_get_contents($src_path); // gets the content of the file and stores it in memory
    $input_f = fopen($src_path, 'r');
    try {
        $myfile = new ProdsFile($account, $dest_path, $verify = false); // creates irods file object
        //read and print out the file
        $myfile->open("w+", $account->default_resc); // creates an empty file on irods server

        while (!feof($input_f)) {
            $myfile->write(fread($input_f, 4096)); // fills the empty file with the content stored in memory on irods server
        }
        //close the file pointer
        $myfile->close();
        fclose($input_f);
    } catch (Exception $e) {
        print_r(var_dump($e));
        if ($e->getCode() == -818000) {
            die("User not allowed to upload iRODS file at this location.");
        } else {
            die("An error occured when trying to upload the file at the specified path.");
        }
        return false;
    }
    $myfile_check = new ProdsFile($account, $dest_path); // creates irods file object to check if it was successfully written
    $fileInfo = $myfile_check->getReplInfo();
    if ($fileInfo[0]['size'] == filesize($src_path)) {
        return true; // in case the file was completely uploaded
    }
    return false;
}
function getIrodsMetadata($filepath, $account = null)
{
    if (!isset($account)) {
        $account = getAccount();
    }
    $myfile = new ProdsFile($account, $filepath);
    $metadata = $myfile->getReplInfo();
    $attributeValueUnits = $myfile->getMeta();
    $AVUsArray = array();
    $sizeOfAVUs = sizeof($attributeValueUnits);
    for ($i = 0; $i < $sizeOfAVUs; $i++) {
        $AVUsArray[$i][0] = $AVUsArray[$i]['name']  = $attributeValueUnits[$i]->name;
        $AVUsArray[$i][1] = $AVUsArray[$i]['value'] = $attributeValueUnits[$i]->value;
        $AVUsArray[$i][2] = $AVUsArray[$i]['units'] = $attributeValueUnits[$i]->units;
    }
    if (!isset($metadata[0])) {
        $metadata[0] = array();
    }
    $metadata = array_merge(
        array('filename' => basename($filepath)),
        array('AVUs' => $AVUsArray), // for AVUs
        $metadata[0]
    );
    return $metadata;
}
function updateAVUs($filepath,
                    $attributeOld,
                    $valueOld,
                    $unitOld,
                    $attributeNew,
                    $valueNew,
                    $unitNew
) {
    try {
        if (existsAVU($filepath, $attributeNew, $valueNew, $unitNew)) {
            addAVU($filepath, $attributeNew, $valueNew, $unitNew);
        } else {
            $account = getAccount();
            if ($account == null) {
                throw new RODSException('Cannot write metadata of file.', 'FILE_WRITE_ERR');
                //die("User not allowed to update the metadata of this file object.");
            }
            $myfile = new ProdsFile($account, $filepath);
            $metaOld = new RODSMeta($attributeOld, $valueOld, $unitOld);
            $metaNew = new RODSMeta($attributeNew, $valueNew, $unitNew);
            $myfile->updateMeta($metaOld, $metaNew);
        }
    } catch (Exception $e) {
        if ($e->getCode() == -818000) {
            die("User not allowed to update the metadata of this file object.");
        } else {
            print_r($e);
        }
    }
}
function existsAVU($filepath, $attribute, $value, $unit)
{
    $metadata = getIrodsMetadata($filepath);
    foreach ($metadata['AVUs'] as $AVU) {
        if ($AVU['name'] == $attribute && $AVU['value'] == $value && $AVU['units'] == $unit) {
            return true;
        }
    }
    return false;
}
function addAVU($filepath, $attributeNew, $valueNew, $unitNew)
{
    if (existsAVU($filepath, $attributeNew, $valueNew, $unitNew)) {
        return 0;
    } else {
        $account = getAccount(true);
        $myfile = new ProdsFile($account, $filepath);
        $metaNew = new RODSMeta($attributeNew, $valueNew, $unitNew);
        $myfile->addMeta($metaNew);
    }
}
function deleteAVU($filepath, $attribute, $value, $unit)
{
    try {
        if (existsAVU($filepath, $attribute, $value, $unit)) {
            $account = getAccount();
            $myfile = new ProdsFile($account, $filepath);
            $meta = new RODSMeta($attribute, $value, $unit);
            $myfile->rmMeta($meta);
        }
    } catch (Exception $e) {
        if ($e->getCode() == -818000) {
            die("User not allowed to delete metadata of this iRODS file.");
        } else {
            print_r($e);
        }
    }
}
function add_irods_avus_to_content_files($filepath, &$text, $article_id)
{
    global $wgDBprefix;
    if (existFile($filepath)) {
        // prepare AVU data to be written
        // files that the article links to
        $local_files = local_files_post($text);
        $irods_files = irods_files_post($text);
        // other links in the article
        $internal_links = internal_links_post($article_id);
        $external_links = external_links_post($article_id);
        // get the category
        $cat = array();
        $sql =  "SELECT cl_to FROM " .
                $wgDBprefix .
                "categorylinks WHERE cl_from='" .
                $article_id .
                "' AND cl_type LIKE 'page'"
        ;
        $result = mysql_query($sql);
        while ($row = mysql_fetch_row($result)) {
            $cat[] = $row[0];
        }
        // get existing irods AVUs
        $existing_avus = getIrodsMetadata($filepath);
        foreach ($existing_avus['AVUs'] as $avu) {
            switch (substr($avu->name, 0, strpos($avu->name, ':'))) {
                case "Local file":
                case "Irods File":
                case "Internal Link":
                case "External Link":
                case "Category":
                    deleteAVU(
                        $filepath,
                        $avu->name,
                        $avu->value,
                        $avu->units
                    );
                    break;
            }
        }
        // add AVUs to the file
        $i = 0;
        foreach ($local_files as $local_file) {
            addAVU($filepath, "Local file: $i", $local_file, "string");
            $i++;
        }
        $i = 0;
        foreach ($irods_files as $irods_file) {
            addAVU($filepath, "Irods File: $i", $irods_file, "string");
            $i++;
        }
        $i = 0;
        foreach($internal_links as $internal_link) {
            addAVU($filepath, "Internal Link: $i", $internal_link, "string");
            $i++;
        }
        $i = 0;
        foreach($external_links as $external_link) {
            addAVU($filepath, "External Link: $i", $external_link, "string");
            $i++;
        }
        if (sizeof($cat) > 1) {
            $sizeof_external_links = sizeof($external_links);
            if (sizeof($cat) < $sizeof_external_links) {
                throw new RODSException();//TODO what exactly is $cat and can it be smaller?
            }
            for ($i = 0; $i < $sizeof_external_links; $i++) {
                if ($cat[$i] != "Pages_with_broken_file_links") {
                    addAVU($filepath, "Category: $i", $cat[$i], "string");
                } elseif (sizeof($cat) == 1) {
                    if (!isset($cat[$i]) || $cat[$i] != "Pages_with_broken_file_links") {
                        addAVU($filepath, 'Category', $cat[0], "string");
                    }
                }
            }
        }
    }
}
function deleteFile($dest_path)
{
    $account = getAccount();
    $myfile = new ProdsFile($account, $dest_path); // creates irods file object
    if ($myfile->exists()) {
        $account_rights = getAccount();
        if ($account != $account_rights) {
            $account = $account_rights;
            $myfile = new ProdsFile($account, $dest_path); // creates irods file object
        } elseif ($account_rights == null) {
            return false;
        }
    }
    try {
        if ($myfile->exists()) {
            $myfile->unlink($account->default_resc); // delete command
        }
        $myfile->close();
    } catch (Exception $e) {
        if ($e->getCode() == -818000) {
            die("User not allowed to delete this iRODS file.");
        } else {
            print_r($e);
        }
    }
}
function writeFile($dest_path, &$content)
{
    $account = getAccount();
    try {
        $myfile = new ProdsFile($account, $dest_path); // creates irods file object
        //read and print out the file
        $myfile->open("w+", $account->default_resc); // creates an empty file on irods server
        $myfile->write($content); // fills the empty file with the content stored in memory on irods server
        //close the file pointer
        $myfile->close();
    } catch (Exception $e) {
        if ($e->getCode() == -818000) {
            die("User not allowed to write iRODS files in this collection.");
        } else {
            print_r($e);
        }
    }
}
function renameFile($file, $new_name)
{
    $account = getAccount();
    try {
        $myfile = new ProdsFile($account, $file); // creates irods file object
        //read and print out the file
        if ($myfile->exists()) {
            $myfile->rename($new_name); // creates an empty file on irods server
        }
        //close the file pointer
        $myfile->close();
    } catch (Exception $e) {
        if ($e->getCode() == -818000) {
            die("User not allowed to rename this iRODS file.");
        } else {
            print_r($e);
        }
    }
    return true;
}
function existFile($file)
{
    $account = getAccount(true);
    // creates irods file object
    $myfile = new ProdsFile($account, $file, $verify=false);
    // checks if the file exists and returns the result
    return $myfile->exists();
}
function existCollection($path)
{
    $account = getAccount();
    $newdir = new ProdsDir($account, $path);
    return $newdir->exists();
}
function get_irods_child_files($path)
{
    $account = getAccount(true);
    $dir = new ProdsDir($account, $path);
    return $dir->getChildFiles();
}
function get_irods_path_info($input_filename, &$full_file_path, &$filename, &$format)
{
    global $configIrodsRootPath;
    $fileinfo = pathinfo($input_filename);
    if (isset($fileinfo['extension'])) {
        $format = $fileinfo['extension'];
    }
    if (substr($input_filename, 0, 1) == '/') {
        $filename = $fileinfo['basename'];
        $full_file_path = $input_filename;
    } else {
        $filename = filename_to_legal_characters(strip_tags(str_replace('\\', ' ', $input_filename)));
        $additional_path = dirname(get_wiki_file_path($filename));
        $full_file_path = "$configIrodsRootPath/$additional_path/$filename";
    }
}
function local_files_post($text)
{
    $matches = array();
    $files_to_upload = array();
    // find all the files that are linked to in this article
    // Old media regular expression:   /\[\[[mM][eE][dD][iI][aA]:(?P<filename>[^|\]]*)[^\]]*?/
    if (preg_match_all("/\[\[[mM][eE][dD][iI][aA]:(?P<filename>[^\|!@\$&\*\?:\[\]\"<>'`=\{\}\\/,;]*)[^\]]*\]\]/", $text, $matches)) {
        $sizeOfFilename = sizeof($matches['filename']);
        for ($i = 0; $i < $sizeOfFilename; $i++) {
            $file = $matches['filename'][$i];
            $files_to_upload[] = $file;
        }
    }
    unset($matches);
    $matches = array();
    // Old image regular expression:   /\[\[[iI][mM][aA][gG][eE]:(?P<filename>[^|\]]*)[^\]]*?/
    if (preg_match_all("/\[\[[iI][mM][aA][gG][eE]:(?P<filename>[^\|!@\$&\*\?:\[\]\"<>'`=\{\}\\/,;]*)[^\]]*\]\]/", $text, $matches)) {
        $sizeOfFilename = sizeof($matches['filename']);
        for ($i = 0; $i < $sizeOfFilename; $i++) {
            $file = $matches['filename'][$i];
            $files_to_upload[] = $file;
        }
    }
    unset($matches);
    $matches = array();
    // Old file regular expression:   /\[\[[fF][iI][lL][eE]:(?P<filename>[^|\]]*)[^\]]*?/
    if (preg_match_all("/\[\[[fF][iI][lL][eE]:(?P<filename>[^\|!@\$&\*\?:\[\]\"<>'`=\{\}\\/,;]*)[^\]]*\]\]/", $text, $matches)) {
        $sizeOfFilename = sizeof($matches['filename']);
        for ($i = 0; $i < $sizeOfFilename; $i++) {
            $file = $matches['filename'][$i];
            $files_to_upload[] = $file;
        }
    }
    return $files_to_upload;
}
function irods_files_post(&$text)
{
    $files_to_upload = array();
    $matches = array();
    if (preg_match_all('/\{\{#[iI][rR][oO][dD][sS]:([^|!@$&*?:"<>\'`=,;]*)[^\}]*\}\}/', $text, $matches)) {
        $sizeOfMatches1 = sizeof($matches[1]);
        for ($i = 0; $sizeOfMatches1; $i++) {
            $file = $matches[1][$i];
            $files_to_upload[] = $file;
        }
    }
    return $files_to_upload;
}
function get_files_based_on_alias($param_alias)
{
    $IrodsAccount = getAccount();
    // look for the mediaWiki Alias
    $flds = array(
        "COL_DATA_NAME"      => null,
        "COL_D_DATA_ID"      => null,
        "COL_DATA_TYPE_NAME" => null,
        "COL_D_RESC_NAME"    => null,
        "COL_DATA_SIZE"      => null,
        "COL_D_OWNER_NAME"   => null,
        "COL_D_OWNER_ZONE"   => null,
        "COL_D_CREATE_TIME"  => null,
        "COL_D_MODIFY_TIME"  => null,
        "COL_COLL_NAME"      => null,
        "COL_D_COMMENTS"     => null
    );
    $select = new RODSGenQueSelFlds(array_keys($flds), array_values($flds));
    $condition = new RODSGenQueConds();
    $condition->add('COL_META_DATA_ATTR_NAME', 'LIKE', 'wiki_alias');
    $condition->add('COL_META_DATA_ATTR_VALUE', 'LIKE', $param_alias);
    $condition->add('COL_COLL_NAME', 'not like', '%/trash/%');
    $condition->add('COL_COLL_NAME', 'like', '/%'); // recursive search
    $conn = RODSConnManager::getConn($IrodsAccount);
    $results = $conn->query($select, $condition, 0, -1);
    RODSConnManager::releaseConn($conn);
    $values = $results->getValues();
    return $values;
}
