<?php

/**
 * Copyright 2013 Drexel University
 */
// class that handles all the hooks
class RodsWikiHooks
{
    // this function is called whenever an article is saved or edited
    // this is executed BEFORE any of the media wiki saving process
    // PURPOSE: saves the wikitext content to a Irods file and adds AVUs with info about it
    public static function onArticleSave(
        &$article,
        &$user,
        &$text,
        &$summary,
        $minor,
        $watchthis,
        $sectionanchor,
        &$flags,
        &$status
    ) {
        global $configSaveArticleSuffix, $configSaveArticleSubdirectory, $configIrodsRootPath;

        require_once dirname(__FILE__) . "/RodsWiki.install.php";
        if (get_class($article) == "WikiFilePage") {
            return false;
        }

        $filename = ($article->mTitle->mTextform) . "." . ((int) ($article->mTitle->mArticleID)) . $configSaveArticleSuffix;

        $content = $text;

        // delete the file if this exists
        deleteFile($configIrodsRootPath . $configSaveArticleSubdirectory . $filename);

        // creates a new file, using the text from the last version:
        writeFile($configIrodsRootPath . $configSaveArticleSubdirectory . $filename, $content);

        // adds irods data into the file storing the wikitext
        add_irods_avus_to_content_files(
            $configIrodsRootPath . $configSaveArticleSubdirectory . $filename,
            $text,
            $article->mTitle->mArticleID
        );

        return true;
    }
    // this function is called when the toolbox is loaded
    // PURPOSE: used to add new links in the toolbox
    public static function ToolboxAddUploadPage(&$sk, &$toolbox)
    {
        global $wgScript, $wgUser;

        if ($wgUser->mId > 0) {// checks if there is an user logged in
            // adds a new link in the toolbox in order to allow the users to access the upload irods page
            $toolbox['specialirodsuploadpage'] = array('href' => $wgScript . '/Special:SpecialIrodsUploadPage', 'id' => 't-SpecialIrodsUploadPage');
        }
        return true;
    }
    // this function is executed whenever an article is deleted
    // PURPOSE: it is used to add a prefix to the wikitext file that is saved on irods
    public static function onArticleDelete(&$article, &$user, &$reason, &$error)
    {
        global $configSaveArticleDeletedPrefix, $configSaveArticleSuffix, $configSaveArticleSubdirectory, $configIrodsRootPath;

        require_once dirname(__FILE__) . "/RodsWiki.install.php";

        if (get_class($article) == "WikiFilePage") {
            return false;
        }

        // if the irods file was never edited after it was published, it will have the following name
        $filename_initial = ($article->mTitle->mTextform) . ".0." . $configSaveArticleSuffix;

        // if the irods file was edited after it was published, it will have the following name
        $filename_revision = ($article->mTitle->mTextform) . "." . ($article->mTitle->mArticleID) . $configSaveArticleSuffix;

        // as this is a deleted file, the new name, including the prefix will be
        $filename_new = $configSaveArticleDeletedPrefix . ($article->mTitle->mTextform) . "." . ($article->mTitle->mArticleID) . $configSaveArticleSuffix;

        // rename the file to add a prefix (2 lines because the file can have initailly 2 filename)
        renameFile(
            $configIrodsRootPath . $configSaveArticleSubdirectory . $filename_initial,
            $configIrodsRootPath . $configSaveArticleSubdirectory . $filename_new,
            false
        );
        renameFile(
            $configIrodsRootPath . $configSaveArticleSubdirectory . $filename_revision,
            $configIrodsRootPath . $configSaveArticleSubdirectory . $filename_new,
            false
        );

        return true;
    }
    // this function is called when an article is restored (after it has been deleted)
    // PURPOSE: remove the prefix of the deleted file, so that it has the initial name
    // 			- in case the file was not in sync with Irods, it will be after restored
    public static function onArticleRevisionUndeleted($title, $revision, $oldPageID)
    {
        global $configSaveArticleDeletedPrefix, $configSaveArticleSuffix, $configSaveArticleSubdirectory, $wgDBprefix, $configIrodsRootPath;

        require_once dirname(__FILE__) . "/RodsWiki.install.php";
        require_once dirname(__FILE__) . '/RodsWiki.functions.php';

        if (get_class($article) == "WikiFilePage") {
            return false;
        }

        // remove the file corresponding with the page that was deleted

        $deleted_filename = $configSaveArticleDeletedPrefix . ($title->mTextform) . "." . $oldPageID . $configSaveArticleSuffix;
        deleteFile($configIrodsRootPath . $configSaveArticleSubdirectory . $deleted_filename);

        // filename of the new file that will be in sync with the undeleted page
        $new_filename = ($title->mTextform) . "." . ($title->mArticleID) . $configSaveArticleSuffix;

        // get the content of the new page from MySQL
        $sql = "SELECT old_text FROM " . $wgDBprefix . "text WHERE old_id='" . ($revision->mTextId) . "'";
        $result = mysql_fetch_row(mysql_query($sql));
        $text = $result[0];

        // saves the file
        writeFile($configIrodsRootPath . $configSaveArticleSubdirectory . $new_filename, $text);

        // add Irods AVUs:
        add_irods_avus_to_content_files(
            $configIrodsRootPath . $configSaveArticleSubdirectory . $new_filename,
            $text,
            ($title->mArticleID)
        );

        return true;
    }
    /*
    public static function CreateFacebookHeadBlock(&$parser, &$text)
    {
        $property = array(
            "http://ogp.me/ns/fb#app_id",
            "http://ogp.me/ns#type",
            "http://ogp.me/ns#url",
            "http://ogp.me/ns#title",
            "http://ogp.me/ns#image",
            "http://ogp.me/ns#description",
            "http://ogp.me/ns/fb/irods_datasets#filesize"
        );
        $content = array(
            "411077478947320",
            "irods_datasets:irods_file",
            "http://" . filter_input(INPUT_SERVER, 'SERVER_NAME') . ':' . filter_input(INPUT_SERVER, 'SERVER_PORT') . filter_input(INPUT_SERVER, 'REQUEST_URI'),
            "Something.prt",
            "http://129.25.20.161/gicl/nsf.jpg",
            "This file is a Pro/E part",
            "50KB"
        );

        for ($i = 0; $i < sizeof($property); $i++) {

            $parser->mOutput->addHeadItem("<meta property=\"$property[$i]\" content=\"$content[$i]\" />\n", $property[$i]);
        }

        return true;
    }
    */
}
