<?php
/**
 * Copyright 2013 Drexel University
 */
class RodsWikiParsers
{
    // Tell MediaWiki that the parser function exists.
    public static function IrodsExtensionSetupParserFunction(&$parser)
    {
        // Create a function hook associating the "irods" magic word with the
        // RodsWikiRenderParserFunction() function.
        $parser->setFunctionHook('irods', 'RodsWikiParsers::RodsWikiRenderParserFunction');
        //$parser->setFunctionHook('irodsqueryavu', 'RodsWikiParsers::RodsWikiRenderQueryAVUFunction');
        $parser->setFunctionHook('irodsshowcollfiles', 'RodsWikiParsers::RodsWikiRenderCollContentsFunction');

        // Return true so that MediaWiki continues to load extensions.
        return true;
    }

    // Render the output of the parser function.
    // Specifically: it renders a link to the special page from this extention that shows metadata of the IRODS file.
    // There the file could be downloaded
    public static function RodsWikiRenderParserFunction($parser, $param1 = '', $param2 = '')//TODO: $parser, $param2 unused?
    {
        global $wgScriptPath, $configIrodsRootPath;
        require_once 'RodsWiki.functions.php';

        $server_name = filter_input(INPUT_SERVER, 'SERVER_NAME');
        $server_port = filter_input(INPUT_SERVER, 'SERVER_PORT');
        $full_path = '';

        // necessary for converted files shortcut tag e.g. {{#irods:hellp.jpg bmp}}
        $params = explode(' ', $param1);
        //$ext = strtolower(pathinfo($params[0], PATHINFO_EXTENSION));//TODO: unused?
        $params[0] = filename_to_legal_characters($params[0]);
        //$fileinfo = pathinfo($params[0]);//TODO: unused?
        if (sizeof($params) < 2) {
            $params[1] = '';
        }

        $result_values = get_files_based_on_alias($params[0]);
        if (array_key_exists('COL_DATA_NAME', $result_values)) {
             $file_count = sizeof($result_values['COL_DATA_NAME']);
        } else {
             $file_count = 0;
        }
        $basename = basename($param1);
        if (substr($param1, 0, 1) == '/') { // check for iRODS full path
            if (existFile($param1)) {
                $output = "[http://$server_name:$server_port$wgScriptPath/index.php/Special:IrodsMetadataPage?filename=$param1 $basename] \n";
                $file_count = 1;
                $full_path = $param1;
            } else {
                $file_count = 0;
                $output = "File at '$param1' was not found.";
            }
            //$search_for_files = false;//TODO: unused?
        } elseif ($file_count > 0) {
            $display_query = true;
        } elseif ($file_count == 0) {
            $wiki_hash_file = $configIrodsRootPath . get_wiki_file_path($param1);
            if (existFile($wiki_hash_file)) {
                $output = "[http://$server_name:$server_port$wgScriptPath/index.php/Special:IrodsMetadataPage?filename=$wiki_hash_file $param1]\n";
                $display_query = false;
                $file_count = 1;
                $full_path = $wiki_hash_file;
            } else {
                $flds = array(
                    "COL_DATA_NAME" => null,
                    "COL_D_DATA_ID" => null,
                    "COL_DATA_TYPE_NAME" => null,
                    "COL_D_RESC_NAME" => null,
                    "COL_DATA_SIZE" => null,
                    "COL_D_OWNER_NAME" => null,
                    "COL_D_OWNER_ZONE" => null,
                    "COL_D_CREATE_TIME" => null,
                    "COL_D_MODIFY_TIME" => null,
                    "COL_COLL_NAME" => null,
                    "COL_D_COMMENTS" => null
                );
                $select = new RODSGenQueSelFlds(array_keys($flds), array_values($flds));

                $condition = new RODSGenQueConds();
                $condition->add('COL_COLL_NAME', 'not like', '%/trash/%');
                $condition->add('COL_DATA_NAME', 'like', '%' . $param1); // recursive search

                $IrodsAccount = getAccount();

                $conn = RODSConnManager::getConn($IrodsAccount);
                $results = $conn->query($select, $condition, 0, -1);
                if ($results == null) {
                    die("No results.");
                }
                RODSConnManager::releaseConn($conn);

                $result_values = $results->getValues();
                $display_query = true;
            }
        }

        if (isset($display_query) && $display_query) {
            # I don't know why this was recalculated here, but it was.
            # Try uncommenting the following if something breaks. ~JO
            #if (array_key_exists('COL_DATA_NAME', $result_values)) {
            #     $file_count = sizeof($result_values['COL_DATA_NAME']);
            #} else {
            #     $file_count = 0;
            #}
            $coll_name = str_replace(' ', '%20', $result_values['COL_COLL_NAME'][0]);
            $data_name = str_replace(' ', '%20', $result_values['COL_DATA_NAME'][0]);
            switch ($file_count) {
                case 0:
                    $output = "File not found.";
                    break;
                case 1:
                    $output = "[http://$server_name$wgScriptPath/index.php/Special:IrodsMetadataPage?filename=$coll_name/$data_name $data_name]\n";
                    $full_path = "$coll_name/$data_name";
                    break;
                default:
                //    $output = "File name or path not unique.";
                    // The following, confusing heredoc is assembling part of a table in WikiText.
                    $output = <<<OUTPUT

                        {|class="wikitable"

                            !colspan="4"|iRODS Files: $file_count
                            |-
                            ! align="left"| Filename
                            ! Size
                            ! Date created
                            ! Date modified
                            |-

OUTPUT;
                    /*
                    $output  = "\n{|class=\"wikitable\"\n"; // beginning of a table
                    $output .= "\n!colspan=\"4\"|iRODS Files: " . $file_count . "\n";
                    $output .= "|- \n";
                    $output .= "! align=\"left\"| Filename \n";
                    $output .= "! Size \n";
                    $output .= "! Date created \n";
                    $output .= "! Date modified \n";
                    $output .= "|- \n";
                    */
                    for ($i = 0; $i < $file_count; $i++) {
                        $output .= "|[http://" .
                            $server_name .
                            $wgScriptPath .
                            "/index.php/Special:IrodsMetadataPage?filename=" .
                            str_replace(' ', '%20', $result_values['COL_COLL_NAME'][$i]) .
                            '/' .
                            str_replace(' ', '%20', $result_values['COL_DATA_NAME'][$i]) .
                            " " .
                            $result_values['COL_DATA_NAME'][$i] .
                            "]\n"
                        ;
                        $output .= "| align=\"right\"|" . number_format($result_values['COL_DATA_SIZE'][$i] / 1024, 1) . " kb\n";
                        $output .= "|&nbsp;&nbsp;&nbsp;" . date("Y-m-d H:i:s", $result_values['COL_D_CREATE_TIME'][$i]) . "&nbsp;&nbsp;&nbsp;\n";
                        $output .= "|&nbsp;&nbsp;&nbsp;" . date("Y-m-d H:i:s", $result_values['COL_D_MODIFY_TIME'][$i]) . "&nbsp;&nbsp;&nbsp;\n";
                        $output .= "|-\n";
                        $full_path[] = str_replace(' ', '%20', $result_values['COL_COLL_NAME'][$i]) . '/' . str_replace(' ', '%20', $result_values['COL_DATA_NAME'][$i]);
                    }
                    $output .= "|}\n"; // beginning of a table

            }
        }

        return array(
            $output,
            'noparse' => false,
            'isHTML' => false,
            'no_files' => $file_count,
            'file_path' => $full_path
        );
    }
    /*
    public static function RodsWikiRenderQueryAVUFunction(
        $parser,
        $param1 = '%%',
        $param2 = '%%',
        $param3 = '%%',
        $param4 = '/%'
    ) {
        global $wgScriptPath;
        require_once 'RodsWiki.functions.php';

        $IrodsAccount = getAccount();
        $server_name = filter_input(INPUT_SERVER, 'SERVER_NAME');

        if (substr($param4, strlen($param4) - 1, 1) == '/') {
            $param4 = substr($param4, 0, strlen($param4) - 1);
        }
        if ($param4 == '') {
            $param4 = "%";
        }

        $flds = array(
            "COL_DATA_NAME" => null,
            "COL_D_DATA_ID" => null,
            "COL_DATA_TYPE_NAME" => null,
            "COL_D_RESC_NAME" => null,
            "COL_DATA_SIZE" => null,
            "COL_D_OWNER_NAME" => null,
            "COL_D_OWNER_ZONE" => null,
            "COL_D_CREATE_TIME" => null,
            "COL_D_MODIFY_TIME" => null,
            "COL_COLL_NAME" => null,
            "COL_D_COMMENTS" => null
        );
        $select = new RODSGenQueSelFlds(array_keys($flds), array_values($flds));
        $condition = new RODSGenQueConds();
        if (!($param1 == '' && $param2 == '' && $param3 == '')) {
            $condition->add('COL_META_DATA_ATTR_NAME', 'LIKE', $param1);
            $condition->add('COL_META_DATA_ATTR_VALUE', 'LIKE', $param2);
            $condition->add('COL_META_DATA_ATTR_UNITS', 'LIKE', $param3);
        }
        $condition->add('COL_COLL_NAME', 'not like', '%/trash/%');
        $condition->add('COL_COLL_NAME', 'like', $param4); // recursive search

        $conn = RODSConnManager::getConn($IrodsAccount);
        $results = $conn->query($select, $condition, 0, -1);
        RODSConnManager::releaseConn($conn);

        $result_values = $results->getValues();
        if (isset($result_values['COL_DATA_NAME']) && sizeof($result_values['COL_DATA_NAME'])) {
            $output = "\n{|class=\"wikitable\"\n"; // beginning of a table
            $output .= "\n!colspan=\"4\"|iRODS Files: " . sizeof($result_values['COL_DATA_NAME']) . "\n";
            $output .= "|- \n";
            $output .= "! align=\"left\"| Filename \n";
            $output .= "! Size \n";
            $output .= "! Date created \n";
            $output .= "! Date modified \n";
            $output .= "|- \n";

            for ($i = 0; $i < sizeof($result_values['COL_DATA_NAME']); $i++) {
                $output .= "|[http://" .
                    $server_name .
                    $wgScriptPath .
                    "/index.php/Special:IrodsMetadataPage?filename=" .
                    str_replace(' ', '%20', $result_values['COL_COLL_NAME'][$i]) .
                    '/' . str_replace(' ', '%20', $result_values['COL_DATA_NAME'][$i]) .
                    " " .
                    $result_values['COL_DATA_NAME'][$i] .
                    "]\n"
                ;
                $output .= "| align=\"right\"|" . number_format($result_values['COL_DATA_SIZE'][$i] / 1024, 1) . " kb\n";
                $output .= "|&nbsp;&nbsp;&nbsp;" . date("Y-m-d H:i:s", $result_values['COL_D_CREATE_TIME'][$i]) . "&nbsp;&nbsp;&nbsp;\n";
                $output .= "|&nbsp;&nbsp;&nbsp;" . date("Y-m-d H:i:s", $result_values['COL_D_MODIFY_TIME'][$i]) . "&nbsp;&nbsp;&nbsp;\n";
                $output .= "|-\n";
            }
            $output .= "|}\n"; // beginning of a table
        } else {
            $output = "No files were found.";
        }

        return array($output, 'noparse' => false, 'isHTML' => false);
    }
    */
    public static function RodsWikiRenderCollContentsFunction($parser, $param1 = '/')
    {
        global $wgScriptPath;
        require_once 'RodsWiki.functions.php';

        $IrodsAccount = getAccount();
        $server_name = filter_input(INPUT_SERVER, 'SERVER_NAME');

        if (substr($param1, strlen($param1) - 1, 1) == '/') {
            $param1 = substr($param1, 0, strlen($param1) - 1);
        }

        $flds = array(
            "COL_DATA_NAME" => null,
            "COL_D_DATA_ID" => null,
            "COL_DATA_TYPE_NAME" => null,
            "COL_D_RESC_NAME" => null,
            "COL_DATA_SIZE" => null,
            "COL_D_OWNER_NAME" => null,
            "COL_D_OWNER_ZONE" => null,
            "COL_D_CREATE_TIME" => null,
            "COL_D_MODIFY_TIME" => null,
            "COL_COLL_NAME" => null,
            "COL_D_COMMENTS" => null
        );
        $select = new RODSGenQueSelFlds(array_keys($flds), array_values($flds));
        $condition = new RODSGenQueConds();
        $condition->add('COL_COLL_NAME', 'like', $param1); // recursive search

        $conn = RODSConnManager::getConn($IrodsAccount);
        $results = $conn->query($select, $condition, 0, -1);
        RODSConnManager::releaseConn($conn);

        $result_values = $results->getValues();

        if (sizeof($result_values['COL_DATA_NAME'])) {
            $output = "\n{|class=\"wikitable\"\n"; // beginning of a table
            $output .= "\n!colspan=\"4\"|iRODS Files: " . sizeof($result_values['COL_DATA_NAME']) . "\n";
            $output .= "|- \n";
            $output .= "! align=\"left\"| Filename \n";
            $output .= "! Size \n";
            $output .= "! Date created \n";
            $output .= "! Date modified \n";
            $output .= "|- \n";
            $sizeOfColDataName = sizeof($result_values['COL_DATA_NAME']);
            for ($i = 0; $i < $sizeOfColDataName; $i++) {
                $output .= "|[http://" .
                    $server_name .
                    $wgScriptPath .
                    "/index.php/Special:IrodsMetadataPage?filename=" .
                    str_replace(' ', '%20', $result_values['COL_COLL_NAME'][$i]) .
                    '/' .
                    str_replace(' ', '%20', $result_values['COL_DATA_NAME'][$i]) .
                    " " .
                    $result_values['COL_DATA_NAME'][$i] .
                    "]\n"
                ;
                $output .= "| align=\"right\"|" . number_format($result_values['COL_DATA_SIZE'][$i] / 1024, 1) . " kb\n";
                $output .= "|&nbsp;&nbsp;&nbsp;" . date("Y-m-d H:i:s", $result_values['COL_D_CREATE_TIME'][$i]) . "&nbsp;&nbsp;&nbsp;\n";
                $output .= "|&nbsp;&nbsp;&nbsp;" . date("Y-m-d H:i:s", $result_values['COL_D_MODIFY_TIME'][$i]) . "&nbsp;&nbsp;&nbsp;\n";
                $output .= "|-\n";
            }
            $output .= "|}\n"; // beginning of a table
        } else {
            $output = "No files were found.";
        }

        return array($output, 'noparse' => false, 'isHTML' => false);
    }
}
