<?php
class SpecialIrodsFileDownload extends SpecialPage
{
    public function __construct()
    {
        $dir = dirname(__FILE__) . '/';
        require_once $dir . "RodsWiki.functions.php";

        parent::__construct('SpecialIrodsFileDownload', '', false);
    }
    public function execute($par)
    {
        ob_start();
        global $configIrodsRootPath;
        $this->setHeaders();
        require_once dirname(__FILE__) . "/RodsWiki.functions.php";
        require_once dirname(__FILE__) . "/RodsWiki.install.php";
        $IrodsAccount = getAccount();
        $load_file_from_local_server = false; // set false to load file from IRODS server; set to load the file directly from the local server
        $filename = filter_input(INPUT_GET, 'filename');
        if (isset($filename)) {
            // extract the tags and trim the input filename
            $file = strip_tags(trim($filename));
            if ($file !== $filename) {
                die('Illegal file name.');
            }
            $exists = filter_input(INPUT_GET, 'exists');
            if (isset($exists)) {
                global $configIrodsRootPath;
                if (existFile($configIrodsRootPath . '/' . get_wiki_file_path($filename))) {
                    echo '1';
                } else {
                    echo '0';
                }
                exit;
            }
        } else {
            $load_file_from_local_server = true;
            $file = "";
        }

        // Gather relevant info about file
        $conversion = filter_input(INPUT_GET, 'conversion');
        if (isset($conversion)) {
            $file_conversion = strip_tags(trim($conversion)); // extract the tags and trim the input filename
            $file_conversion = basename($file_conversion); // extract the name of the file, without preceding path
            //$file_extension = strtolower(substr(strrchr($file_conversion, "."), 1)); // extract the extention
        }
        // extract the name of the file, without preceding path
        $filebasename = basename($file);
        //$finfo = finfo_open(FILEINFO_MIME_TYPE);//TODO this is broken
        //$ctype = finfo_file($finfo, $filebasename);//TODO
        //finfo_close($finfo);
        if ($load_file_from_local_server == false) {
            // create a file object, specifing the path of the file to be read
            if (!isset($conversion)) {
                if (substr($file, 0, 1) == "/") {
                    $full_file_path = $file;
                } else {
                    $full_file_path = $configIrodsRootPath . get_wiki_file_path($filebasename);
                }
            } else {
                $full_file_path = $configIrodsRootPath . get_wiki_file_path($filebasename, 0) . "conv-" . filename_to_legal_characters($filebasename) . '/' . $file_conversion;
            }

            $myfile = new ProdsFile($IrodsAccount, $full_file_path);

            //check if the file exists
            if ($myfile->exists()) {
                try {
                    //Begin writing headers
                    header("Pragma: public");
                    header("Expires: 0");
                    header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
                    header("Cache-Control: public");
                    header("Content-Description: File Transfer");
                    //Use the switch-generated Content-Type
                    header("Content-Type: application/octet-stream");//$ctype");
                    //Force the download
                    if (isset($conversion)) {
                        $header = "Content-Disposition: attachment; filename=" . $file_conversion . ";";
                    } else {
                        $header = "Content-Disposition: attachment; filename=" . $filebasename . ";";
                    }
                    header($header);
                    header("Content-Transfer-Encoding: binary");
                    header('Last-Modified: ' . date('r'));
                    $myfile->open("r"); // open in secure mode
                    $fileinfo = $myfile->getStats();
                    header("Content-Length: {$fileinfo->size}");
                    ob_clean();
                    flush();
                    // move the cursor to the begining of the file
                    $myfile->rewind();
                    // streaming chunks
                    // Variable is updated at every 1MB read from the server
                    while ($str = $myfile->read(10485760)) {
                        print($str);
                        ob_flush();
                        flush();
                    }
                    ob_end_clean();

                    $myfile->close();
                } catch (Exception $e) {
                    if ($e->getCode() == -818000) {
                        die("This iRODS account has no rights to read this file.");
                    }
                }
            } else { // in case the file was not found on the IRODS Server
                $load_file_from_local_server = true;
            }
        } else { // in case there is no connection to the IRODS Server
            $load_file_from_local_server = true;
            // in case the file was not loaded from the IRODS server, look on the local server
            if (file_exists("../../images/" . get_wiki_file_path($filebasename)) && isset($filename)) { // check if the file exists on the local server
                //Begin writing headers
                header("Pragma: public");
                header("Expires: 0");
                header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
                header("Cache-Control: public");
                header("Content-Description: File Transfer");
                //Use the switch-generated Content-Type
                header("Content-Type: application/octet-stream");//$ctype");
                //Force the download
                $header = "Content-Disposition: attachment; filename=" . $filebasename . ";";
                header($header);
                header("Content-Transfer-Encoding: binary");
                header("Content-Length: " . filesize($file));
                header('Last-Modified: ' . date('r'));
                ob_clean();
                flush();
                // The PDF source is in original.pdf
                echo file_get_contents("../../images/" . get_wiki_file_path($filebasename));
            }
        }
    }
}
