<?php

/**
 * Copyright 2013 Drexel University
 */
// SPECIAL PAGE: This page is an upload form that saves a file directly on the IRODS server (no storage on the local server)

class SpecialIrodsUploadPage extends SpecialPage
{
    public function __construct()
    {
        require_once dirname(__FILE__) . "/RodsWiki.functions.php";
        require_once dirname(__FILE__) . "/RodsWiki.install.php";
        parent::__construct('SpecialIrodsUploadPage');
    }
    public function execute($par)
    {
        global $wgOut, $wgScriptPath, $configIrodsRootPath, $wgUser;
        $this->setHeaders();
        //$options = filter_input(INPUT_POST, 'options');

        //OpenRodsWiki($IrodsAccount);
        if ($wgUser->mId == 0) { // checks if there is a user logged in
            $wgOut->setPagetitle("Restricted access");
            $wgOut->addHTML("You must be logged in to have access to this page.");
            return;
        }

        // get the maximum file size
        $max_upload = (ini_get('upload_max_filesize'));
        $max_post = (ini_get('post_max_size'));
        $memory_limit = (ini_get('memory_limit'));
        $max_file_upload = min($max_upload, $max_post, $memory_limit);

        // this block uploads the file to irods
        $success = 0;
        $filepath = filter_input(INPUT_POST, 'filename');
        if (isset($_FILES['uploadedfile'])) { // it is executed only if the form was submitted already
            $filename = basename($_FILES['uploadedfile']['name']);
            if (!existFile($configIrodsRootPath . '/' . get_wiki_file_path($filename))) {
                if ($filepath != "") {
                    $filename = basename(get_wiki_file_path($filepath));
                }
            }
            if (preg_match('/\.[0-9]*$/i', $filename)) {
                $pathinfo = pathinfo($filename);
                $filename = $pathinfo['filename'];
            }
            $dest_file = $configIrodsRootPath . '/' . get_wiki_file_path($filename);
            $md5 = md5($filename);
            $directoryName1 = substr($md5, 0, 1);
            $directoryName2 = substr($md5, 0, 2);
            $directory1 = makeIrodsDirectory($configIrodsRootPath, $directoryName1);
            $directory2 = makeIrodsChildDirectory($directory1, $directoryName2);
            if (uploadFileOnIrods($_FILES['uploadedfile']['tmp_name'], $dest_file)) {
                $add_mediawiki_alias = filter_input(INPUT_POST, 'add_mediawiki_alias');
                if (isset($add_mediawiki_alias)) {
                    addAVU($dest_file, 'wiki_alias', basename($dest_file), '');
                }
                $success = 1;
            }
        }
        $server_name = filter_input(INPUT_SERVER, 'SERVER_NAME');
        $server_port = server_port_for_url(filter_input(INPUT_SERVER, 'SERVER_PORT'));
        $request_uri = filter_input(INPUT_SERVER, 'REQUEST_URI');
        if ($success) {
            $redirect = "http://{$server_name}{$server_port}{$wgScriptPath}/index.php/Special:IrodsMetadataPage?filename={$filename}";
            die("<script>alert('File successfully uploaded!'); window.location = '$redirect';</script><META HTTP-EQUIV='refresh' CONTENT=\"0; URL='$redirect'\">");
        }
        $get_filename = filter_input(INPUT_GET, 'filename');
        if (!$success) {
            $wgOut->addHTML("To include a file in a page, use a link in one of the following form: <br /><pre>{{#irods:" . ((isset($get_filename)) ? ($get_filename) : 'File.jpg') . "}}</pre>");
        }

        // shows the upload form, no matter if the file was uploaded or not
        $wgOut->addWikiText("== Source file ==");
        if (isset($get_filename)) {
            $filenameString = ' value="' . $get_filename . '"';
        } else {
            $filenameString = '';
        }
        $fileExists = (int) isset($get_filename);
        $output = <<<OUTPUT
    <form enctype="multipart/form-data" action="http://$server_name$server_port$request_uri" method="POST" name="uploadform">
        <div>
            <p>Choose a file to upload: <input name="uploadedfile" type="file" size = "50" onchange="change_filename(this.value); update_mediawiki_url(this.value);"></p>
        </div>
        <div>
            <p>File name: <input readonly="readonly" type="text" name="filename" id="filename" onkeyup="onNameChange(this.value);update_mediawiki_url(this.value)" size="50" $filenameString></p>
            <p>(change this if you want to upload this file with another name on Irods)</p>
        </div>
        <div>Maximum file size: $max_file_upload</div>
            <div id="file_exists" style="display: none">
                <font color="red">&#9888; Warning: This file already exists!</font>
            </div>
            <div id="file_safe" style="display: none">
                <font color="green">&#10003; This file could be uploaded safely!</font>
            </div>
            <script>
                configIrodsRootPath = "$configIrodsRootPath";
                wikiBaseUrl = "http://$server_name$server_port$wgScriptPath";
                filenameLocked = $fileExists;
            </script>
            <script src="$wgScriptPath/extensions/RodsWiki/md5.js"></script>
            <script src="$wgScriptPath/extensions/RodsWiki/SpecialUpload.js"></script>
            <div>
                <p>Add MediaWiki Alias <input type="checkbox" name="add_mediawiki_alias" id="add_mediawiki_alias" checked="checked"></p>
                <p><i>(An alias allows you to refer to this file using the filename only, if the alias is file name is unique)</i></p>
            </div>
            <div>
                <p>iRODS upload path: <input disabled="disabled" type="text" size="50" name="irodspath" id="irodspath"></p>
            </div>
        </div>
        <div id="results"></div>
        <input type="submit" value="Upload File" name="upload_button" id="upload_button"/>
    </form>
OUTPUT;
        $wgOut->addHTML($output);
        $wgOut->setPagetitle("Upload mediafile");
    }
}
